//
//  KSSDKObject.h
//  GenericObject
//
//  Created by Sunil Phani Manne on 30/09/16.
//  Copyright © 2016 Sunil Phani Manne. All rights reserved.
//

#import <UIKit/UIKit.h>

#import "KSConstants.h"
#import "ISyncableObject.h"
#import "KSObjectMetadata.h"
#import "KSSDKObjectRecord.h"
#import "IOfflineCapabilities.h"
#import "ICRUDAble.h"

@interface KSSDKObject : NSObject <ISyncableObject, IOfflineCapabilities ,ICRUDAble >

//Object level info
@property (nonatomic, copy) NSString *name;
@property (nonatomic, copy) NSString *nameSpaceName;
@property (nonatomic, readonly) NSString *fullyQualifiedName;

@property (nonatomic, readonly) NSString *originalTablefullyQualifiedName;
@property (nonatomic, readonly) NSString *historyTablefullyQualifiedName;

@property (nonatomic, assign) KSSDKObjectMode mode;
@property (nonatomic, readonly) NSArray <KSSDKObjectRecord *> *records;

//Attributes required for Sync server processing
@property (nonatomic, assign) KSSDKObjectAction action;

//Metadata object
@property (nonatomic, weak) KSObjectMetadata *metadata;

@property (nonatomic, readonly) NSString *objectServiceName;

/**
 * An Array of SDKObjects
 * Used in upload and download scenarios
 */
@property (nonatomic, strong) NSArray <KSSDKObject *> *sdkObjectsForUploadandDownload;

- (instancetype) init __attribute__((unavailable("Must use initWithName: or initWithName:andNamespace: instead.")));

- (instancetype)initWithName:(NSString *)name;

- (instancetype)initWithName:(NSString *)name
                       error:(OfflineObjectsError **)error;

- (instancetype)initWithName:(NSString *)name
            andNamespaceName:(NSString *)namespaceName
                       error:(OfflineObjectsError **)error;

- (instancetype)initWithObjectServiceName:(NSString *)objectServiceName
                                andObject:(NSString *)name
                         andNamespaceName:(NSString *)namespaceName
                                    error:(OfflineObjectsError **)error;

- (void)addRecord:(KSSDKObjectRecord *)record;
- (void)addRecords:(NSArray <KSSDKObjectRecord *> *)records;
- (void)removeRecord:(KSSDKObjectRecord *)record;
- (void)removeAllRecords;


- (NSArray *)recordsWithAction:(KSSDKObjectRecordAction)recordAction;

//TODO: Implement a KVO listener and automatically update the classified records when the records are update...
//Miscellaneous methods
- (NSDictionary *)recordsClassifedWithRecordAction;

/**
 * Add the given pk value pair to the given object
 * @param primaryKeyValue will be added to a Set which maintains the primaryKeyValuePairs for the records
 */
- (void) setSetOfPrimaryKeyValuePairs:(NSDictionary<NSString *,id> *)primaryKeyValue;

/**
 * Gives the Set which maintains the primaryKeyValuePairs for the records
 */
- (NSSet<NSDictionary<NSString *,id> *> *) getSetOfPrimaryKeyValuePairs;

//Online methods
//TODO: Think about online methods

/**
 * Rollback an Object to its previous sync state
 * @param primaryKeyValueMap of record only for which rollback should happen.
 * @param error which has to be passes incase of rollback failure.
 * @return Yes if rollback is success else No.
 */
- (BOOL) rollback:(NSDictionary *)primaryKeyValueMap
            error:(OfflineObjectsError **)error;

@end
