Ext.data.JsonP.android_net_PskKeyManager({"tagname":"class","name":"android.net.PskKeyManager","autodetected":{},"files":[{"filename":"PskKeyManager.js","href":"PskKeyManager.html#android-net-PskKeyManager"}],"extends":"java.lang.Object","members":[{"name":"MAX_IDENTITY_HINT_LENGTH_BYTES","tagname":"property","owner":"android.net.PskKeyManager","id":"property-MAX_IDENTITY_HINT_LENGTH_BYTES","meta":{}},{"name":"MAX_IDENTITY_LENGTH_BYTES","tagname":"property","owner":"android.net.PskKeyManager","id":"property-MAX_IDENTITY_LENGTH_BYTES","meta":{}},{"name":"MAX_KEY_LENGTH_BYTES","tagname":"property","owner":"android.net.PskKeyManager","id":"property-MAX_KEY_LENGTH_BYTES","meta":{}},{"name":"chooseClientKeyIdentity","tagname":"method","owner":"android.net.PskKeyManager","id":"method-chooseClientKeyIdentity","meta":{}},{"name":"chooseServerKeyIdentityHint","tagname":"method","owner":"android.net.PskKeyManager","id":"method-chooseServerKeyIdentityHint","meta":{}},{"name":"getKey","tagname":"method","owner":"android.net.PskKeyManager","id":"method-getKey","meta":{}}],"alternateClassNames":[],"aliases":{},"id":"class-android.net.PskKeyManager","short_doc":"Provider of key material for pre-shared key (PSK) key exchange used in TLS-PSK cipher suites. ...","classIcon":"icon-class","superclasses":["java.lang.Object"],"subclasses":[],"mixedInto":[],"mixins":[],"parentMixins":[],"requires":[],"uses":[],"html":"<div><pre class=\"hierarchy\"><h4>Hierarchy</h4><div class='subclass first-child'>java.lang.Object<div class='subclass '><strong>android.net.PskKeyManager</strong></div></div><h4>Files</h4><div class='dependency'><a href='source/PskKeyManager.html#android-net-PskKeyManager' target='_blank'>PskKeyManager.js</a></div></pre><div class='doc-contents'><p>Provider of key material for pre-shared key (PSK) key exchange used in TLS-PSK cipher suites.</p>\n\n<p> <h3>Overview of TLS-PSK</h3></p>\n\n<p> <p>TLS-PSK is a set of TLS/SSL cipher suites which rely on a symmetric pre-shared key (PSK) to\n secure the TLS/SSL connection and mutually authenticate its peers. These cipher suites may be\n a more natural fit compared to conventional public key based cipher suites in some scenarios\n where communication between peers is bootstrapped via a separate step (for example, a pairing\n step) and requires both peers to authenticate each other. In such scenarios a symmetric key (PSK)\n can be exchanged during the bootstrapping step, removing the need to generate and exchange public\n key pairs and X.509 certificates.</p></p>\n\n<p> <p>When a TLS-PSK cipher suite is used, both peers have to use the same key for the TLS/SSL\n handshake to succeed. Thus, both peers are implicitly authenticated by a successful handshake.\n This removes the need to use a {@code TrustManager} in conjunction with this {@code KeyManager}.\n </p></p>\n\n<p> <h3>Supporting multiple keys</h3></p>\n\n<p> <p>A peer may have multiple keys to choose from. To help choose the right key, during the\n handshake the server can provide a <em>PSK identity hint</em> to the client, and the client can\n provide a <em>PSK identity</em> to the server. The contents of these two pieces of information\n are specific to application-level protocols.</p></p>\n\n<p> <p><em>NOTE: Both the PSK identity hint and the PSK identity are transmitted in cleartext.\n Moreover, these data are received and processed prior to peer having been authenticated. Thus,\n they must not contain or leak key material or other sensitive information, and should be\n treated (e.g., parsed) with caution, as untrusted data.</em></p></p>\n\n<p> <p>The high-level flow leading to peers choosing a key during TLS/SSL handshake is as follows:\n <ol>\n <li>Server receives a handshake request from client.\n <li>Server replies, optionally providing a PSK identity hint to client.</li>\n <li>Client chooses the key.</li>\n <li>Client provides a PSK identity of the chosen key to server.</li>\n <li>Server chooses the key.</li>\n </li></ol></p></p>\n\n<p> <p>In the flow above, either peer can signal that they do not have a suitable key, in which case\n the the handshake will be aborted immediately. This may enable a network attacker who does not\n know the key to learn which PSK identity hints or PSK identities are supported. If this is a\n concern then a randomly generated key should be used in the scenario where no key is available.\n This will lead to the handshake aborting later, due to key mismatch -- same as in the scenario\n where a key is available -- making it appear to the attacker that all PSK identity hints and PSK\n identities are supported.</p></p>\n\n<p> <h3>Maximum sizes</h3></p>\n\n<p> <p>The maximum supported sizes are as follows:\n <ul>\n <li>256 bytes for keys (see <a href=\"#!/api/android.net.PskKeyManager-property-MAX_KEY_LENGTH_BYTES\" rel=\"android.net.PskKeyManager-property-MAX_KEY_LENGTH_BYTES\" class=\"docClass\">MAX_KEY_LENGTH_BYTES</a>),</li>\n <li>128 bytes for PSK identity and PSK identity hint (in modified UTF-8 representation) (see\n <a href=\"#!/api/android.net.PskKeyManager-property-MAX_IDENTITY_LENGTH_BYTES\" rel=\"android.net.PskKeyManager-property-MAX_IDENTITY_LENGTH_BYTES\" class=\"docClass\">MAX_IDENTITY_LENGTH_BYTES</a> and <a href=\"#!/api/android.net.PskKeyManager-property-MAX_IDENTITY_HINT_LENGTH_BYTES\" rel=\"android.net.PskKeyManager-property-MAX_IDENTITY_HINT_LENGTH_BYTES\" class=\"docClass\">MAX_IDENTITY_HINT_LENGTH_BYTES</a>).</li>\n </ul></p></p>\n\n<p> <h3>Subclassing</h3>\n Subclasses should normally provide their own implementation of {@code getKey} because the default\n implementation returns no key, which aborts the handshake.</p>\n\n<p> <h3>Known issues</h3>\n The implementation of {@code ECDHE_PSK} cipher suites in API Level 21 contains a bug which breaks\n compatibility with other implementations. {@code ECDHE_PSK} cipher suites are enabled by default\n on platforms with API Level 21 when an {@code SSLContext} is initialized with a\n {@code PskKeyManager}. A workaround is to disable {@code ECDHE_PSK} cipher suites on platforms\n with API Level 21.</p>\n\n<p> <h3>Example</h3>\n The following example illustrates how to create an {@code SSLContext} which enables the use of\n TLS-PSK in {@code SSLSocket}, {@code SSLServerSocket} and {@code SSLEngine} instances obtained\n from it.\n <pre> {@code\n PskKeyManager pskKeyManager = ...;</pre></p>\n\n<p> SSLContext sslContext = SSLContext.getInstance(\"TLS\");\n sslContext.init(\n         new KeyManager[] &#123;pskKeyManager&#125;,\n         new TrustManager[0], // No TrustManagers needed for TLS-PSK\n         null // Use the default source of entropy\n         );</p>\n\n<p> SSLSocket sslSocket = (SSLSocket) sslContext.getSocketFactory().createSocket(...);\n }</p>\n</div><div class='members'><div class='members-section'><div class='definedBy'>Defined By</div><h3 class='members-title icon-property'>Properties</h3><div class='subsection'><div id='property-MAX_IDENTITY_HINT_LENGTH_BYTES' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.net.PskKeyManager'>android.net.PskKeyManager</span><br/><a href='source/PskKeyManager.html#android-net-PskKeyManager-property-MAX_IDENTITY_HINT_LENGTH_BYTES' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.net.PskKeyManager-property-MAX_IDENTITY_HINT_LENGTH_BYTES' class='name expandable'>MAX_IDENTITY_HINT_LENGTH_BYTES</a> : String<span class=\"signature\"></span></div><div class='description'><div class='short'>Maximum supported length (in bytes) for PSK identity hint (in modified UTF-8 representation). ...</div><div class='long'><p>Maximum supported length (in bytes) for PSK identity hint (in modified UTF-8 representation).</p>\n<p>Defaults to: <code>&quot;null&quot;</code></p></div></div></div><div id='property-MAX_IDENTITY_LENGTH_BYTES' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.net.PskKeyManager'>android.net.PskKeyManager</span><br/><a href='source/PskKeyManager.html#android-net-PskKeyManager-property-MAX_IDENTITY_LENGTH_BYTES' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.net.PskKeyManager-property-MAX_IDENTITY_LENGTH_BYTES' class='name expandable'>MAX_IDENTITY_LENGTH_BYTES</a> : String<span class=\"signature\"></span></div><div class='description'><div class='short'>Maximum supported length (in bytes) for PSK identity (in modified UTF-8 representation). ...</div><div class='long'><p>Maximum supported length (in bytes) for PSK identity (in modified UTF-8 representation).</p>\n<p>Defaults to: <code>&quot;null&quot;</code></p></div></div></div><div id='property-MAX_KEY_LENGTH_BYTES' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.net.PskKeyManager'>android.net.PskKeyManager</span><br/><a href='source/PskKeyManager.html#android-net-PskKeyManager-property-MAX_KEY_LENGTH_BYTES' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.net.PskKeyManager-property-MAX_KEY_LENGTH_BYTES' class='name expandable'>MAX_KEY_LENGTH_BYTES</a> : String<span class=\"signature\"></span></div><div class='description'><div class='short'>Maximum supported length (in bytes) for PSK. ...</div><div class='long'><p>Maximum supported length (in bytes) for PSK.</p>\n<p>Defaults to: <code>&quot;null&quot;</code></p></div></div></div></div></div><div class='members-section'><div class='definedBy'>Defined By</div><h3 class='members-title icon-method'>Methods</h3><div class='subsection'><div id='method-chooseClientKeyIdentity' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.net.PskKeyManager'>android.net.PskKeyManager</span><br/><a href='source/PskKeyManager.html#android-net-PskKeyManager-method-chooseClientKeyIdentity' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.net.PskKeyManager-method-chooseClientKeyIdentity' class='name expandable'>chooseClientKeyIdentity</a>( <span class='pre'>identityHint</span> ) : String<span class=\"signature\"></span></div><div class='description'><div class='short'>Gets the PSK identity to report to the server to help agree on the PSK for the provided\n engine. ...</div><div class='long'><p>Gets the PSK identity to report to the server to help agree on the PSK for the provided\n engine.</p>\n\n<p> <p>\n The default implementation returns an empty string.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>identityHint</span> : String<div class='sub-desc'><p>identity hint provided by the server or {@code null} if none provided.</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'>String</span><div class='sub-desc'><p>PSK identity to provide to the server. {@code null} is permitted but will be\n         converted into an empty string.</p>\n</div></li></ul></div></div></div><div id='method-chooseServerKeyIdentityHint' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.net.PskKeyManager'>android.net.PskKeyManager</span><br/><a href='source/PskKeyManager.html#android-net-PskKeyManager-method-chooseServerKeyIdentityHint' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.net.PskKeyManager-method-chooseServerKeyIdentityHint' class='name expandable'>chooseServerKeyIdentityHint</a>( <span class='pre'></span> ) : String<span class=\"signature\"></span></div><div class='description'><div class='short'>Gets the PSK identity hint to report to the client to help agree on the PSK for the provided\n engine. ...</div><div class='long'><p>Gets the PSK identity hint to report to the client to help agree on the PSK for the provided\n engine.</p>\n\n<p> <p>\n The default implementation returns {@code null}.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'>String</span><div class='sub-desc'><p>PSK identity hint to be provided to the client or {@code null} to provide no hint.</p>\n</div></li></ul></div></div></div><div id='method-getKey' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.net.PskKeyManager'>android.net.PskKeyManager</span><br/><a href='source/PskKeyManager.html#android-net-PskKeyManager-method-getKey' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.net.PskKeyManager-method-getKey' class='name expandable'>getKey</a>( <span class='pre'>identityHint, identity</span> ) : Object {javax.crypto.SecretKey}<span class=\"signature\"></span></div><div class='description'><div class='short'>Gets the PSK to use for the provided engine. ...</div><div class='long'><p>Gets the PSK to use for the provided engine.</p>\n\n<p> <p>\n The default implementation returns {@code null}.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>identityHint</span> : String<div class='sub-desc'><p>identity hint provided by the server to help select the key or\n        {@code null} if none provided.</p>\n</div></li><li><span class='pre'>identity</span> : String<div class='sub-desc'><p>identity provided by the client to help select the key.</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'>Object {javax.crypto.SecretKey}</span><div class='sub-desc'><p>key or {@code null} to signal to peer that no suitable key is available and to abort\n         the handshake.</p>\n</div></li></ul></div></div></div></div></div></div></div>","meta":{}});