Ext.data.JsonP.android_os_StrictMode({"tagname":"class","name":"android.os.StrictMode","autodetected":{},"files":[{"filename":"StrictMode.js","href":"StrictMode.html#android-os-StrictMode"}],"extends":"java.lang.Object","members":[{"name":"NETWORK_POLICY_ACCEPT","tagname":"property","owner":"android.os.StrictMode","id":"property-NETWORK_POLICY_ACCEPT","meta":{}},{"name":"NETWORK_POLICY_LOG","tagname":"property","owner":"android.os.StrictMode","id":"property-NETWORK_POLICY_LOG","meta":{}},{"name":"NETWORK_POLICY_REJECT","tagname":"property","owner":"android.os.StrictMode","id":"property-NETWORK_POLICY_REJECT","meta":{}},{"name":"allowThreadDiskReads","tagname":"method","owner":"android.os.StrictMode","id":"method-allowThreadDiskReads","meta":{}},{"name":"allowThreadDiskWrites","tagname":"method","owner":"android.os.StrictMode","id":"method-allowThreadDiskWrites","meta":{}},{"name":"enableDefaults","tagname":"method","owner":"android.os.StrictMode","id":"method-enableDefaults","meta":{}},{"name":"getThreadPolicy","tagname":"method","owner":"android.os.StrictMode","id":"method-getThreadPolicy","meta":{}},{"name":"getVmPolicy","tagname":"method","owner":"android.os.StrictMode","id":"method-getVmPolicy","meta":{}},{"name":"noteSlowCall","tagname":"method","owner":"android.os.StrictMode","id":"method-noteSlowCall","meta":{}},{"name":"setThreadPolicy","tagname":"method","owner":"android.os.StrictMode","id":"method-setThreadPolicy","meta":{}},{"name":"setVmPolicy","tagname":"method","owner":"android.os.StrictMode","id":"method-setVmPolicy","meta":{}}],"alternateClassNames":[],"aliases":{},"id":"class-android.os.StrictMode","short_doc":"StrictMode is a developer tool which detects things you might be\n doing by accident and brings them to your attention...","classIcon":"icon-class","superclasses":["java.lang.Object"],"subclasses":[],"mixedInto":[],"mixins":[],"parentMixins":[],"requires":[],"uses":[],"html":"<div><pre class=\"hierarchy\"><h4>Hierarchy</h4><div class='subclass first-child'>java.lang.Object<div class='subclass '><strong>android.os.StrictMode</strong></div></div><h4>Files</h4><div class='dependency'><a href='source/StrictMode.html#android-os-StrictMode' target='_blank'>StrictMode.js</a></div></pre><div class='doc-contents'><p><p>StrictMode is a developer tool which detects things you might be\n doing by accident and brings them to your attention so you can fix\n them.</p>\n\n<p> <p>StrictMode is most commonly used to catch accidental disk or\n network access on the application's main thread, where UI\n operations are received and animations take place.  Keeping disk\n and network operations off the main thread makes for much smoother,\n more responsive applications.  By keeping your application's main thread\n responsive, you also prevent\n <a href=\"{@docRoot}guide/practices/design/responsiveness.html\">ANR dialogs</a>\n from being shown to users.</p>\n\n<p> <p class=\"note\">Note that even though an Android device's disk is\n often on flash memory, many devices run a filesystem on top of that\n memory with very limited concurrency.  It's often the case that\n almost all disk accesses are fast, but may in individual cases be\n dramatically slower when certain I/O is happening in the background\n from other processes.  If possible, it's best to assume that such\n things are not fast.</p></p>\n\n<p> <p>Example code to enable from early in your\n android.app.Application, android.app.Activity, or\n other application component's\n android.app.Application.onCreate method:</p>\n\n<p><pre> public void onCreate() {\n     if (DEVELOPER_MODE) {\n         StrictMode.setThreadPolicy(new android.os.StrictMode.ThreadPolicy.Builder()\n                 .detectDiskReads()\n                 .detectDiskWrites()\n                 .detectNetwork()   // or .detectAll() for all detectable problems\n                 .penaltyLog()\n                 .build());\n         StrictMode.setVmPolicy(new <a href=\"#!/api/android.os.StrictMode.VmPolicy.Builder\" rel=\"android.os.StrictMode.VmPolicy.Builder\" class=\"docClass\">StrictMode.android.os.StrictMode.VmPolicy.Builder</a>()\n                 .detectLeakedSqlLiteObjects()\n                 .detectLeakedClosableObjects()\n                 .penaltyLog()\n                 .penaltyDeath()\n                 .build());\n     }\n     super.onCreate();\n }\n </pre></p>\n\n<p> <p>You can decide what should happen when a violation is detected.\n For example, using android.os.StrictMode.ThreadPolicy.Builder.penaltyLog you can\n watch the output of <code>adb logcat</code> while you use your\n application to see the violations as they happen.</p>\n\n<p> <p>If you find violations that you feel are problematic, there are\n a variety of tools to help solve them: threads, <a href=\"#!/api/android.os.Handler\" rel=\"android.os.Handler\" class=\"docClass\">android.os.Handler</a>,\n <a href=\"#!/api/android.os.AsyncTask\" rel=\"android.os.AsyncTask\" class=\"docClass\">android.os.AsyncTask</a>, android.app.IntentService, etc.\n But don't feel compelled to fix everything that StrictMode finds.  In particular,\n many cases of disk access are often necessary during the normal activity lifecycle.  Use\n StrictMode to find things you did by accident.  Network requests on the UI thread\n are almost always a problem, though.</p>\n\n<p> <p class=\"note\">StrictMode is not a security mechanism and is not\n guaranteed to find all disk or network accesses.  While it does\n propagate its state across process boundaries when doing\n <a href=\"#!/api/android.os.Binder\" rel=\"android.os.Binder\" class=\"docClass\">android.os.Binder</a> calls, it's still ultimately a best\n effort mechanism.  Notably, disk or network access from JNI calls\n won't necessarily trigger it.  Future versions of Android may catch\n more (or fewer) operations, so you should never leave StrictMode\n enabled in applications distributed on Google Play.</p>\n</div><div class='members'><div class='members-section'><div class='definedBy'>Defined By</div><h3 class='members-title icon-property'>Properties</h3><div class='subsection'><div id='property-NETWORK_POLICY_ACCEPT' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-property-NETWORK_POLICY_ACCEPT' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-property-NETWORK_POLICY_ACCEPT' class='name expandable'>NETWORK_POLICY_ACCEPT</a> : String<span class=\"signature\"></span></div><div class='description'><div class='short'>{@hide} ...</div><div class='long'><p>{@hide}</p>\n<p>Defaults to: <code>&quot;0&quot;</code></p></div></div></div><div id='property-NETWORK_POLICY_LOG' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-property-NETWORK_POLICY_LOG' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-property-NETWORK_POLICY_LOG' class='name expandable'>NETWORK_POLICY_LOG</a> : String<span class=\"signature\"></span></div><div class='description'><div class='short'>{@hide} ...</div><div class='long'><p>{@hide}</p>\n<p>Defaults to: <code>&quot;1&quot;</code></p></div></div></div><div id='property-NETWORK_POLICY_REJECT' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-property-NETWORK_POLICY_REJECT' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-property-NETWORK_POLICY_REJECT' class='name expandable'>NETWORK_POLICY_REJECT</a> : String<span class=\"signature\"></span></div><div class='description'><div class='short'>{@hide} ...</div><div class='long'><p>{@hide}</p>\n<p>Defaults to: <code>&quot;2&quot;</code></p></div></div></div></div></div><div class='members-section'><div class='definedBy'>Defined By</div><h3 class='members-title icon-method'>Methods</h3><div class='subsection'><div id='method-allowThreadDiskReads' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-allowThreadDiskReads' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-allowThreadDiskReads' class='name expandable'>allowThreadDiskReads</a>( <span class='pre'></span> ) : Object {android.os.StrictMode.ThreadPolicy}<span class=\"signature\"></span></div><div class='description'><div class='short'>A convenience wrapper that takes the current\n android.os.StrictMode.ThreadPolicy from getThreadPolicy, modifies it\n t...</div><div class='long'><p>A convenience wrapper that takes the current\n <a href=\"#!/api/android.os.StrictMode.ThreadPolicy\" rel=\"android.os.StrictMode.ThreadPolicy\" class=\"docClass\">android.os.StrictMode.ThreadPolicy</a> from <a href=\"#!/api/android.os.StrictMode-method-getThreadPolicy\" rel=\"android.os.StrictMode-method-getThreadPolicy\" class=\"docClass\">getThreadPolicy</a>, modifies it\n to permit disk reads, and sets the new policy\n with <a href=\"#!/api/android.os.StrictMode-method-setThreadPolicy\" rel=\"android.os.StrictMode-method-setThreadPolicy\" class=\"docClass\">setThreadPolicy</a>, returning the old policy so you\n can restore it at the end of a block.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'>Object {android.os.StrictMode.ThreadPolicy}</span><div class='sub-desc'><p>the old policy, to be passed to setThreadPolicy to\n         restore the policy.</p>\n</div></li></ul></div></div></div><div id='method-allowThreadDiskWrites' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-allowThreadDiskWrites' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-allowThreadDiskWrites' class='name expandable'>allowThreadDiskWrites</a>( <span class='pre'></span> ) : Object {android.os.StrictMode.ThreadPolicy}<span class=\"signature\"></span></div><div class='description'><div class='short'>A convenience wrapper that takes the current\n android.os.StrictMode.ThreadPolicy from getThreadPolicy, modifies it\n t...</div><div class='long'><p>A convenience wrapper that takes the current\n <a href=\"#!/api/android.os.StrictMode.ThreadPolicy\" rel=\"android.os.StrictMode.ThreadPolicy\" class=\"docClass\">android.os.StrictMode.ThreadPolicy</a> from <a href=\"#!/api/android.os.StrictMode-method-getThreadPolicy\" rel=\"android.os.StrictMode-method-getThreadPolicy\" class=\"docClass\">getThreadPolicy</a>, modifies it\n to permit both disk reads &amp; writes, and sets the new policy\n with <a href=\"#!/api/android.os.StrictMode-method-setThreadPolicy\" rel=\"android.os.StrictMode-method-setThreadPolicy\" class=\"docClass\">setThreadPolicy</a>, returning the old policy so you\n can restore it at the end of a block.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'>Object {android.os.StrictMode.ThreadPolicy}</span><div class='sub-desc'><p>the old policy, to be passed to <a href=\"#!/api/android.os.StrictMode-method-setThreadPolicy\" rel=\"android.os.StrictMode-method-setThreadPolicy\" class=\"docClass\">setThreadPolicy</a> to\n         restore the policy at the end of a block</p>\n</div></li></ul></div></div></div><div id='method-enableDefaults' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-enableDefaults' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-enableDefaults' class='name expandable'>enableDefaults</a>( <span class='pre'></span> )<span class=\"signature\"></span></div><div class='description'><div class='short'>Enable the recommended StrictMode defaults, with violations just being logged. ...</div><div class='long'><p>Enable the recommended StrictMode defaults, with violations just being logged.</p>\n\n<p> <p>This catches disk and network access on the main thread, as\n well as leaked SQLite cursors and unclosed resources.  This is\n simply a wrapper around <a href=\"#!/api/android.os.StrictMode-method-setVmPolicy\" rel=\"android.os.StrictMode-method-setVmPolicy\" class=\"docClass\">setVmPolicy</a> and <a href=\"#!/api/android.os.StrictMode-method-setThreadPolicy\" rel=\"android.os.StrictMode-method-setThreadPolicy\" class=\"docClass\">setThreadPolicy</a>.</p>\n</div></div></div><div id='method-getThreadPolicy' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-getThreadPolicy' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-getThreadPolicy' class='name expandable'>getThreadPolicy</a>( <span class='pre'></span> )<span class=\"signature\"></span></div><div class='description'><div class='short'>Returns the current thread's policy. ...</div><div class='long'><p>Returns the current thread's policy.</p>\n</div></div></div><div id='method-getVmPolicy' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-getVmPolicy' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-getVmPolicy' class='name expandable'>getVmPolicy</a>( <span class='pre'></span> )<span class=\"signature\"></span></div><div class='description'><div class='short'>Gets the current VM policy. ...</div><div class='long'><p>Gets the current VM policy.</p>\n</div></div></div><div id='method-noteSlowCall' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-noteSlowCall' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-noteSlowCall' class='name expandable'>noteSlowCall</a>( <span class='pre'>name</span> )<span class=\"signature\"></span></div><div class='description'><div class='short'>For code to note that it's slow. ...</div><div class='long'><p>For code to note that it's slow.  This is a no-op unless the\n current thread's <a href=\"#!/api/android.os.StrictMode.ThreadPolicy\" rel=\"android.os.StrictMode.ThreadPolicy\" class=\"docClass\">android.os.StrictMode.ThreadPolicy</a> has\n android.os.StrictMode.ThreadPolicy.Builder.detectCustomSlowCalls\n enabled.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>name</span> : String<div class='sub-desc'><p>a short string for the exception stack trace that's\n             built if when this fires.</p>\n</div></li></ul></div></div></div><div id='method-setThreadPolicy' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-setThreadPolicy' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-setThreadPolicy' class='name expandable'>setThreadPolicy</a>( <span class='pre'>policy</span> )<span class=\"signature\"></span></div><div class='description'><div class='short'>Sets the policy for what actions on the current thread should\n be detected, as well as the penalty if such actions oc...</div><div class='long'><p>Sets the policy for what actions on the current thread should\n be detected, as well as the penalty if such actions occur.</p>\n\n<p> <p>Internally this sets a thread-local variable which is\n propagated across cross-process IPC calls, meaning you can\n catch violations when a system service or another process\n accesses the disk or network on your behalf.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>policy</span> : Object {StrictMode.ThreadPolicy}<div class='sub-desc'><p>the policy to put into place</p>\n</div></li></ul></div></div></div><div id='method-setVmPolicy' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='android.os.StrictMode'>android.os.StrictMode</span><br/><a href='source/StrictMode.html#android-os-StrictMode-method-setVmPolicy' target='_blank' class='view-source'>view source</a></div><a href='#!/api/android.os.StrictMode-method-setVmPolicy' class='name expandable'>setVmPolicy</a>( <span class='pre'>policy</span> )<span class=\"signature\"></span></div><div class='description'><div class='short'>Sets the policy for what actions in the VM process (on any\n thread) should be detected, as well as the penalty if suc...</div><div class='long'><p>Sets the policy for what actions in the VM process (on any\n thread) should be detected, as well as the penalty if such\n actions occur.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>policy</span> : Object {StrictMode.VmPolicy}<div class='sub-desc'><p>the policy to put into place</p>\n</div></li></ul></div></div></div></div></div></div></div>","meta":{}});